local addonName = ...
local guildName = nil
local db = nil
local guildMaster = nil
local pName = UnitName("player")

local today = nil
local sessionTotal = 0
local allTimeTotal = nil
-- Our total looted last time we did a broadcast
local lastTransmitTime = nil

------ Locale
local L_SESSION = "This session"
local L_ALLTIME = "All time total"
local L_NOINCOME = "You haven't generated any income to your guild for the past month - stop slacking!"
local L_NOINCOME_GM = "You haven't generated any income to your guild for the past month - stop slacking! %s is very disappointed."
local L_GOODBOY = "Good boy. Once you provide some more income, daddy will give you something special."
local L_GOODBOY_GM = "Good boy. Once you provide some more income, daddy %s will give you something special."
------

local function sortOnTotal(a, b)
	local aT = a == pName and allTimeTotal or BGMOthersDB[guildName][a]
	local bT = b == pName and allTimeTotal or BGMOthersDB[guildName][b]
	return aT > bT
end

local tmpSort = {}
local scoreFormat = "%d. %s"
local function updateTooltip(tt)
	tt:AddLine("Guild Money")
	local added = nil
	if sessionTotal > 0 then
		tt:AddDoubleLine("This session", GetCoinTextureString(sessionTotal), 1, 1, 1, 0, 1, 0)
		added = true
	end
	if db[today] and db[today] > 0 then
		tt:AddDoubleLine(HONOR_TODAY, GetCoinTextureString(db[today]), 1, 1, 1, 0, 1, 0)
		added = true
	end
	local week = 0
	for i = today, (today - 7), -1 do
		week = week + (db[i] or 0)
	end
	if week > 0 then
		tt:AddDoubleLine(CALENDAR_REPEAT_WEEKLY, GetCoinTextureString(week), 1, 1, 1, 0, 1, 0)
		added = true
	end
	local month = week
	for i = (today - 7), (today - 30), -1 do
		month = month + (db[i] or 0)
	end
	if month > 0 then
		tt:AddDoubleLine(CALENDAR_REPEAT_MONTHLY, GetCoinTextureString(month), 1, 1, 1, 0, 1, 0)
		added = true
	end
	if allTimeTotal and allTimeTotal > 0 then
		tt:AddDoubleLine(L_ALLTIME, GetCoinTextureString(allTimeTotal), 1, 1, 1, 0, 1, 0)
		added = true
	end

	wipe(tmpSort)
	tmpSort[1] = pName
	for k, v in pairs(BGMOthersDB[guildName]) do
		tmpSort[#tmpSort + 1] = k
	end
	if #tmpSort > 1 then
		added = true
		table.sort(tmpSort, sortOnTotal)
		tt:AddLine(" ")
		tt:AddLine("Top 5 contributors")
		for i, p in next, tmpSort do
			local v = p == pName and allTimeTotal or BGMOthersDB[guildName][p]
			tt:AddDoubleLine(scoreFormat:format(i, p), GetCoinTextureString(v), 1, 1, 1, 0, 1, 0)
		end
	end

	if not added then
		if guildMaster then
			tt:AddLine(L_NOINCOME_GM:format(guildMaster), 0.2, 1, 0.2, 1)
		else
			tt:AddLine(L_NOINCOME, 0.2, 1, 0.2, 1)
		end
	else
		tt:AddLine(" ")
		if guildMaster then
			tt:AddLine(L_GOODBOY_GM:format(guildMaster), 0.2, 1, 0.2, 1)
		else
			tt:AddLine(L_GOODBOY, 0.2, 1, 0.2, 1)
		end
	end
end

local ldb = LibStub("LibDataBroker-1.1")
local bGuildMoney = ldb:NewDataObject("Reagents", {
	type = "data source",
	icon = "Interface\\MoneyFrame\\UI-GoldIcon",
	text = "Guild money",
	OnTooltipShow = updateTooltip,
})
--local goldString = GOLD_AMOUNT:gsub("%%d", ""):trim()
--local silverString = SILVER_AMOUNT:gsub("%%d", ""):trim()
--local copperString = COPPER_AMOUNT:gsub("%%d", ""):trim()
local function updateText()
	local amount = db[today] or 0
	if amount == 0 then
		bGuildMoney.text = "Guild money"
	else
		local gold = floor(math.abs(amount) / 10000)
		local silver = mod(floor(math.abs(amount) / 100), 100)
		local copper = mod(floor(math.abs(amount)), 100)
		if gold ~= 0 then
			bGuildMoney.text = format("%dg%ds%dc", gold, silver, copper)
		elseif silver ~= 0 then
			bGuildMoney.text = format("%ds%dc", silver, copper)
		else
			bGuildMoney.text = format("%dc", copper)
		end
	end
end

local matchGold = GOLD_AMOUNT:gsub("%%d", "(%%d+)")
local matchSilver = SILVER_AMOUNT:gsub("%%d", "(%%d+)")
local matchCopper = COPPER_AMOUNT:gsub("%%d", "(%%d+)")
local personalMoney = YOU_LOOT_MONEY:gsub("%%s", "(.+)")
local guildMoney = YOU_LOOT_MONEY_GUILD:gsub("%(", "%%("):gsub("%)", "%%)"):gsub("%%s", "(.+)")
local guildMoneyGroup = LOOT_MONEY_SPLIT_GUILD:gsub("%(", "%%("):gsub("%)", "%%)"):gsub("%%s", "(.+)")

local function getTotalFromString(msg)
	local g = (msg:match(matchGold) or 0) * 1000
	local s = (msg:match(matchSilver) or 0) * 100
	local c = msg:match(matchCopper) or 0
	return g + s + c
end

local function scanForGM()
	for i = 1, GetNumGuildMembers(true) do
		local n, _, r = GetGuildRosterInfo(i)
		if n and r == 0 then
			if n ~= pName then guildMaster = n end
			break
		end
	end
end

local f = CreateFrame("Frame")
f:RegisterEvent("ADDON_LOADED")
f:RegisterEvent("GUILD_ROSTER_UPDATE")
f:RegisterEvent("CHAT_MSG_ADDON")
f:SetScript("OnEvent", function(self, event, addon, message, channel, sender)
	if event == "ADDON_LOADED" then
		if addon ~= addonName then return end
		guildName = GetGuildInfo("player")
		if not guildName then
			print("It seems you're not in a guild; Broker_GuildMoney won't work.")
			return
		end
		BGMDB = BGMDB or {}
		BGMTotalDB = BGMTotalDB or {}
		BGMOthersDB = BGMOthersDB or {}
		if not BGMDB[guildName] then BGMDB[guildName] = {} end
		if not BGMTotalDB[guildName] then BGMTotalDB[guildName] = 0 end
		if not BGMOthersDB[guildName] then BGMOthersDB[guildName] = {} end
		db = BGMDB[guildName]
		
		BGMOthersDB[guildName].Glexx = 100000
		BGMOthersDB[guildName].Tubbygold = 20000

		-- Borrowed from MoneyFu (probably ckk)
		local serverHour, serverMinute = GetGameTime()
		local utcHour = tonumber(date("!%H"))
		local utcMinute = tonumber(date("!%M"))
		local ser = serverHour + serverMinute / 60
		local utc = utcHour + utcMinute / 60
		local offset = floor((ser - utc) * 2 + 0.5) / 2
		if offset >= 12 then
			offset = offset - 24
		elseif offset < -12 then
			offset = offset + 24
		end
		today = floor((time() / 60 / 60 + offset) / 24)
		
		-- Anything older than 30 days is added to the total and reset
		local oneMonth = today - 30
		for k, v in pairs(db) do
			if k < oneMonth then
				BGMTotalDB[guildName] = BGMTotalDB[guildName] + v
				db[k] = nil
			end
		end
		allTimeTotal = BGMTotalDB[guildName]
		for k, v in pairs(db) do
			allTimeTotal = allTimeTotal + v
		end
		SendAddonMessage(addonName, tostring(allTimeTotal), "GUILD")

		scanForGM()
		updateText()
		self:UnregisterEvent("ADDON_LOADED")
	elseif event == "GUILD_ROSTER_UPDATE" then
		scanForGM()
	elseif event == "CHAT_MSG_ADDON" then
		if addon ~= addonName or sender == pName then return end
		BGMOthersDB[guildName][sender] = tonumber(message)
	end
end)

local function filter(self, event, msg, ...)
	-- |not db| catches failed startup
	if not db then return end
	local personal, guild = msg:match(guildMoney)
	if not guild or GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then
		personal, guild = msg:match(guildMoneyGroup)
	end
	if not guild then return end

	local looted = (getTotalFromString(guild) or 0)
	sessionTotal = sessionTotal + looted
	db[today] = (db[today] or 0) + looted
	allTimeTotal = allTimeTotal + looted

	local t = GetTime()
	if not lastTransmitTime or (t > lastTransmitTime + 120) then
		SendAddonMessage(addonName, tostring(allTimeTotal), "GUILD")
		lastTransmitTime = t
	end

	updateText()

	local t = GetCoinTextureString(getTotalFromString(personal))
	return false, YOU_LOOT_MONEY:format(t), ...
end

ChatFrame_AddMessageEventFilter("CHAT_MSG_MONEY", filter)

